/**
 * Called when the "post link" dialog box is created. This retrieves the
 * default values for the form parameters from the URL that specifies the
 * box itself. See for example postCurrentPage() in deliciousOverlay.js.
 */
function loadArgs()
{
    var vars=window.arguments;

    var url = "";
    var description = "";
    var extended = "";
    var tags = "";
    var pair;
    for(i=0;i<=vars.length-1;i++)
    {
        pair=vars[i].split("=");
        if(pair[0]=="url")
            url=decodeURIComponent(pair[1]);
        else if(pair[0]=="description")
            description=decodeURIComponent(pair[1]);
        else if(pair[0]=="extended")
            extended=decodeURIComponent(pair[1]);
        else if(pair[0]=="tags")
            tags=decodeURIComponent(pair[1]);
    }

    document.getElementById("url").value = url;
    document.getElementById("description").value = description;
    document.getElementById("extended").value = extended;
    document.getElementById("tags").value = tags;

    var wordsInExtended = extended.replace(/\s+/g, ' ').replace(/^\W+|\W+$/g, '').split(' ').length;
    document.getElementById("extended-row").setAttribute("flex", wordsInExtended/2);

    document.getElementById('extended').focus();
    
    var users = userManager.getUsers();
    var defaultUser = userManager.getDefaultUser();
    var postAsPopup = document.getElementById('use-account-popup');
    for (i in users) {
        var item = document.createElement("menuitem");
        item.setAttribute('label', userManager.getAccountNameForUser(users[i]));
        item.setAttribute('value', users[i]);
        var node = postAsPopup.appendChild(item);
        if (users[i] == defaultUser) {
            document.getElementById('use-account-list').selectedItem = item;
        }
    }

    showLoadingStatus("tags-recommended");
    showLoadingStatus("tags-your");
    showLoadingStatus("tags-popular");
	loadExperimentalFields(url, description);
}

// Globals, needed to retain some info through the asynchronous loading process.
var g_url = '';
var g_username = '';
var g_password = '';
var g_description = '';

/*
 * Load del.icio.us' experimental post page, parse tags from it and populate
 * the dialog's "experimental" field with them.
 */
function loadExperimentalFields(url, description)
{
   	g_username = deliciousService.getUsername();
   	if (g_username.length == 0) {
   	    window.close();
   	    return;
   	}
    g_password = deliciousService.getPassword();
   	if (g_password.length == 0) {
   	    window.close();
   	    return;
   	}
    g_url= url;
    g_description = description;
    
	var queryURL = "http://del.icio.us/new/" + encodeURIComponent(g_username) +
		"?v=2&url=" + encodeURIComponent(g_url) +
		"&title=" + encodeURIComponent(g_description);
    try {
        asyncGetURL(queryURL, g_username, g_password, false, processExperimentalResponse);
    } catch(e) {
        showExperimentalError("Getting experimental data failed. ", e);
    }
}

//
// Callback for the asynchronous getting of the experimental fields info.
//
function processExperimentalResponse(resp)
{
    if (resp != null) {
        delPostDivs = resp.replace(/\n/g, "").split(/<div class="delPost">/).slice(1);
        delPostDivs[2] = delPostDivs[2].split(/<\/div>\s*<\/div>/)[0] + "</div></div>";

        var tagsRecommended = parseTags(delPostDivs[0]);
        var tagsYour = parseTags(delPostDivs[1]);
        var tagsPopular = parseTags(delPostDivs[2]);
    
        writeTagsToDialogElement(tagsRecommended, "tags-recommended", null);
        writeTagsToDialogElement(tagsYour, "tags-your", tagsRecommended);
        writeTagsToDialogElement(tagsPopular, "tags-popular", null);

        var r = document.getElementById("tags-recommended");
        if (tagsRecommended.length == 0) {
            var label = document.createElement("label");
            label.setAttribute("value", "No recommended tags found.");
            label.setAttribute("style", "font-style: italic;");
            r.appendChild(label);
        } else {
            document.getElementById("tags-recommended-row").setAttribute("flex", tagsRecommended.length);
        }
        var r = document.getElementById("tags-your");
        if (tagsYour.length == 0) {
            var label = document.createElement("label");
            label.setAttribute("value", "You have no tags defined.");
            label.setAttribute("style", "font-style: italic;");
            r.appendChild(label);
        } else {
            document.getElementById("tags-your-row").setAttribute("flex", tagsYour.length);
        }
        var r = document.getElementById("tags-popular");
        if (tagsPopular.length == 0) {
            var label = document.createElement("label");
            label.setAttribute("value", "No popular tags found.");
            label.setAttribute("style", "font-style: italic;");
            r.appendChild(label);
        } else {
            document.getElementById("tags-popular-row").setAttribute("flex", tagsPopular.length);
        }
    }
}

//
// Indicate an error in loading the experimental fields.
//
function showExperimentalError(explanation, exception)
{
    var r = document.getElementById("tags-recommended");
    r.value = explanation + exception;
    var r = document.getElementById("tags-your");
    r.value = explanation + exception;
    var r = document.getElementById("tags-popular");
    r.value = explanation + exception;
}    

//
// Write text into a tag field indicating that the field is being updated.
//
function showLoadingStatus(elementId)
{
    clearLabelsFromElement(elementId);
    var label = document.createElement("label");
    label.setAttribute("value", "Loading tags from del.icio.us...");
    label.setAttribute("style", "font-style: italic;");
    document.getElementById(elementId).appendChild(label);
}

//
// Clear any existing tags or text (labels) from a tag field.
//
function clearLabelsFromElement(elementId)
{
    var element = document.getElementById(elementId);
    var labels = element.getElementsByTagName('label');
    for (var i = labels.length - 1; i >= 0; --i) {
        element.removeChild(labels.item(i));
    }
}

/**
 * Store tags to a XULElement.
 */
function writeTagsToDialogElement(tags, elementId, tagsToEmphasise)
{
    clearLabelsFromElement(elementId);
    var element = document.getElementById(elementId);
    for (i in tags) {
        var label = document.createElement("label");
        label.setAttribute("value", tags[i]);
        label.setAttribute("onclick", "tagClicked(this);");
        for (j in tagsToEmphasise) {
        	if (tagsToEmphasise[j] == tags[i]) {
                label.setAttribute("style", "color: red;");
            }
        }
        element.appendChild(label);
    }
}

/**
 * Parse tags from del.icio.us response.
 * This is highly dependent on their HTML format. Hope they don't change their HTML often.
 */
function parseTags(tagDivHTML, element)
{
    re = /swapTag\('(.*?)'/i;
    var i = 0;
    tags = new Array;
    match = tagDivHTML.match(re);
    while (match != null) {
        tags.push(match[1]);
        i += match.index + match[0].length;
        match = tagDivHTML.slice(i).match(re);
    }
    return tags;
}

/**
 * Switch clicked tags on or off.
 */
function tagClicked(tag)
{
    var tags = document.getElementById("tags").value.split(' ');
    var present = false;
    for (t in tags) {
        if (tags[t].toLowerCase() == tag.value.toLowerCase()) {
            tags.splice(t,1);
            present = true;
        }
    }
    if (!present) {
        tags.push(tag.value);
    }
    document.getElementById("tags").value = tags.join(' ');
}

/**
 * Change the account to post to a different one.
 */
function changeAccount(event)
{
    showLoadingStatus("tags-recommended");
    showLoadingStatus("tags-your");
    showLoadingStatus("tags-popular");
    userManager.setDefaultUser(event.target.value, onChangeAccountComplete);
}

//
// Callback for async login: continue with loading the experimental fields.
//
function onChangeAccountComplete(req)
{
	loadExperimentalFields(document.getElementById("url").value, document.getElementById("description").value);
}

/**
 * Executed when user confirms the "post link" dialog box.
 */
function accept()
{
    var url = document.getElementById("url").value;
    var description = document.getElementById("description").value;
    var extended = document.getElementById("extended").value;
    var tags = document.getElementById("tags").value;
    var date = new Date();

    var request = deliciousService.postURL(url, description, extended, tags, date.toISO8601String());
    if (!request || request.status != 200) {
        return false;
    }
    return true;
}

/**
 * Format a date as required to make a del.icio.us post (the &dt= parameter).
 * This uses ISO 8601 format: "CCYY-MM-DDThh:mm:ssZ"
 * See here for details: http://www.cl.cam.ac.uk/~mgk25/iso-time.html
 * Example: 1984-09-01T14:21:31Z
 */
Date.prototype.toISO8601String = function()
{
    var str;
    
    // date
    var year = this.getUTCFullYear().toString();
    var month = (this.getUTCMonth()+1).toString();
    month = month.length < 2 ? ("0" + month) :  month;
    
    var day = this.getUTCDate().toString();
    day = day.length < 2 ? ("0" + day) : day;
    
    var hours = this.getUTCHours().toString();
    hours = hours.length < 2 ? ("0" + hours) : hours;
    
    var minutes = this.getUTCMinutes().toString();
    minutes = minutes.length < 2 ? ("0" + minutes) : minutes;
    
    var seconds = this.getUTCSeconds().toString();
    seconds = seconds.length < 2 ? ("0" + seconds) : seconds;
    
    return year + "-" + month + "-" + day + "T" + hours + ":" + minutes + ":" + seconds + "Z";
}

/**
 * Parse an ISO 8601 formatted string into a Date object.
 */
Date.prototype.fromISO8601String = function(str)
{
    str = str.replace("Z","");
    var parts = str.split("T");
    var dateArr = parts[0].split("-");
    var timeArr = parts[1].split(":");
    
    this.setUTCFullYear(dateArr[0]);
    this.setUTCMonth(dateArr[1]-1);
    this.setUTCDate(dateArr[2]);
    this.setUTCHours(timeArr[0]);
    this.setUTCMinutes(timeArr[1]);
    this.setUTCSeconds(timeArr[2]);
}

